@@ -1,6 +1,6 @@
 import sqlalchemy as sa
 from sqlalchemy_utils.functions import create_database
-from pyg_base import cache, cfg_read, as_list, dictable, Dict, is_dict, is_dictable, is_strs, is_str, is_int, is_date, dt2str, ulist, try_back, unique
+from pyg_base import cache, cfg_read, as_list, dictable, replace, Dict, is_dict, is_dictable, is_strs, is_str, is_int, is_date, dt2str, ulist, try_back, unique
 from pyg_encoders import as_reader, as_writer, dumps, loads
 from sqlalchemy import Table, Column, Integer, String, MetaData, Identity, Float, DATE, DATETIME, TIME, select, func, not_, desc, asc
 from sqlalchemy.orm import Session
@@ -82,7 +82,7 @@ def _pairs2connection(*pairs, **connection):
                 k = k.strip()
                 v = v.strip().replace("'","")
                 connection[k] = v
-    connection = {k.lower() : v.replace(' ','+').replace('{','').replace('}','') for k, v in connection.items() if v is not None}
+    connection = {k.lower() : replace(replace(v, ' ','+'),['{','}'],'') for k, v in connection.items() if v is not None}
     return connection
 
 def _db(connection):
@@ -981,10 +981,22 @@ class sql_cursor(object):
         else:        
             return self._read_row(rows, reader = reader, columns = columns, load = load)
 
-    def docs(self, start = None, stop = None, step = None):
+    def docs(self, start = None, stop = None, step = None, reader = None):
         rows = self._read_statement(start = start, stop = stop, step = step)
-        docs = self._rows_to_docs(rows)
+        docs = self._rows_to_docs(rows, reader = reader)
         return dictable(docs)
+    
+    def read(self, item = 0, reader = None):
+        value = len(self) + item if item < 0 else item
+        statement = self.statement()
+        if self.order is None:
+            row = list(self.engine.connect().execute(statement.limit(value+1)))[value]
+        else:
+            row = list(self.engine.connect().execute(statement.offset(value).limit(1)))[0]
+        doc = self._rows_to_docs(row, reader = reader)
+        rtn = self._undock(doc)
+        return rtn
+
                 
     def __getitem__(self, value):
         """
@@ -1014,22 +1026,19 @@ class sql_cursor(object):
 
         elif isinstance(value, slice):
             start, stop, step = value.start, value.stop, value.step
+            if step is False:
+                step = None
+                reader = False
+            else:
+                reader = None
             rows = self._read_statement(start = start, stop = stop, step = step)
-            docs = self._rows_to_docs(rows)
+            docs = self._rows_to_docs(rows, reader = reader)
             columns = self.columns
             res = dictable([self._undock(doc, columns = columns) for doc in docs])
             return res
 
         elif is_int(value):
-            value = len(self) + value if value < 0 else value
-            statement = self.statement()
-            if self.order is None:
-                row = list(self.engine.connect().execute(statement.limit(value+1)))[value]
-            else:
-                row = list(self.engine.connect().execute(statement.offset(value).limit(1)))[0]
-            doc = self._rows_to_docs(row)
-            rtn = self._undock(doc)
-            return rtn
+            return self.read(value)
     
     def update_one(self, doc, upsert = True):
         """
